from abstract_ocr import *
from abstract_utilities import *
images = '''/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/1.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/2.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/3.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/4.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/5.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/6.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/7.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/8.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/10.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/11.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/12.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/13.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/14.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/15.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/16.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/17.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/18.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/19.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/20.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/21.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/22.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/23.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/24.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/25.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/26.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/27.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/28.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/29.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/30.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/31.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/32.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/33.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/34.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/35.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/36.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/37.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/38.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/39.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/40.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/41.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/42.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/43.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/44.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/45.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/46.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/47.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/48.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/49.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/50.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/51.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/52.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/53.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/54.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/55.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/56.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/57.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/58.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/59.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/60.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/61.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/62.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/63.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/64.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/65.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/66.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/68.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/69.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/70.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/71.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/72.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/73.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/74.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/76.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/77.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/78.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/79.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/80.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/81.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/82.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/83.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/84.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/85.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/86.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/87.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/88.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/89.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/90.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/91.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/92.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/93.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/94.jpg
/home/computron/Desktop/downloaded_site/epsteinsblackbook.com/epsteinsblackbook.com/black-book-images/95.jpg'''
images = images.split('\n')
def is_number_in_text(text):
   for char in text:
       if is_number(char):
           return True
   return False
curr_dir = os.getcwd()
file_path = os.path.join(curr_dir,'the_names.json')
data=[]
for image in images:
    text = convert_image_to_text(image)
    texts = text.split('\n\n')
    for text in texts:
        for line in text.split('\n'):
            if line:
                if is_number_in_text(line):
                    if data ==[]:
                        data.append([])
                    data[-1].append(line)
                else:
                    data.append([line])
    safe_dump_to_file(data=data,file_path=file_path)
safe_dump_to_file(data=data,file_path=file_path)



from ..dependencies import *
#WindowManagement
def readme_event_check(self)->bool:
    """
    Checks if the Generate README event has been triggered.

    The Generate README event is triggered when a user opts to generate a README file for the prompt.

    Returns:
        bool: True if the Generate README event has been triggered, otherwise False
    """
    if self.event=="-GENERATE_README-":
        prompt_number=self.get_prompt_data_section_number()
        prompt_data = self.prompt_data_list[prompt_number]
        request_data = self.request_data_list[prompt_number]
        
        files_list = self.window['-FILES_LIST_FILES-'].Values
        result,files_list=read_me_window(files_list)

        request_data+='\n\n'+result
        for file_path in files_list:
            if os.path.isfile(file_path):
                file_contents = read_from_file(file_path)
                self.chunk_title= os.path.basename(file_path)
                prompt_data+=self.add_to_chunk(file_contents)
                
        self.chunk_type="CODE"
        
        if prompt_data:
            self.update_prompt_data(prompt_data)
        
        if request_data:
            self.update_request_data(request_data)
    else:
        return False
    return True
def database_event_check(self)->bool:
    """
    Checks if the Generate README event has been triggered.

    The Generate README event is triggered when a user opts to generate a README file for the prompt.

    Returns:
        bool: True if the Generate README event has been triggered, otherwise False
    """
    if self.event=="-GENERATE_DATABASE_QUERY-":
        prompt_number=self.get_prompt_data_section_number()
        prompt_data = self.prompt_data_list[prompt_number]
        request_data = self.request_data_list[prompt_number]
        prompt_data+='\n\n'+get_instruction_from_tableName()
        self.update_prompt_data(prompt_data)
        
        contents = self.read_file_contents(file_path)  # Assuming read_file_contents is defined to handle different file types
        contents = safe_json_loads(contents)


        
        if prompt_data:
            self.update_prompt_data(prompt_data)
        
        if request_data:
            self.update_request_data(request_data)
    else:
        return False
    return True

def perform_event_checks(self)->None:
    """
    Iterates through the different event checks to see if any has been triggered.

    The function stops checking once an event has been triggered.
    """
    for event_check in self.event_check_keys:
        # Use getattr to get the method by name, then call it
        method = getattr(self, event_check)
        self.bool_loop = method()
        
        # If you want to stop checking once bool_loop is True (or some condition), you can break here
        if self.bool_loop or self.event==None:  # Assuming self.bool_loop should stop the loop when True
            break
    

def browser_event_query(self,event:str=None,values:dict=None,window=None):
    values=values or self.window_mgr.get_values()
    event=event or self.window_mgr.get_event()
    window=window or self.window
    self.browser_mgr.handle_event(event,values,window)

    
def while_window(self,event,values,window):
    self.event,self.values,window=event,values,window
    if self.event != None:
        if self.loop_one == False:
            
            self.update_all()
            self.restore_instruction_defaults()
            self.update_instruction_mgr()
            self.window_mgr.update_value(key="-INSTRUCTIONS_TEXT-",value=self.default_instructions)
            for browser_event in ['-SCAN_FILES-',
                                  '-SCAN_RESPONSES-']:
                self.browser_event_query(browser_event,self.values,self.window)
            
        self.next_read_mgr.execute_queue()
        self.script_event_js = get_event_key_js(event = self.event,key_list=self.additions_key_list)
        self.perform_event_checks()
        self.loop_one=True



# pyqt_gui.py

from PyQt5 import QtCore, QtGui, QtWidgets
from abstract_ai import GptManager

class ProgressFrame(QtWidgets.QGroupBox):
    def __init__(self, parent=None):
        super().__init__("PROGRESS", parent)
        layout = QtWidgets.QHBoxLayout()

        # Progress Text (read-only QLineEdit)
        self.progress_text = QtWidgets.QLineEdit("Awaiting Prompt")
        self.progress_text.setObjectName("-PROGRESS_TEXT-")
        self.progress_text.setReadOnly(True)
        self.progress_text.setFixedHeight(24)
        self.progress_text.setMinimumWidth(200)

        # Progress Bar
        self.progress_bar = QtWidgets.QProgressBar()
        self.progress_bar.setObjectName("-PROGRESS-")
        self.progress_bar.setRange(0, 100)
        self.progress_bar.setFixedHeight(24)
        self.progress_bar.setMaximumWidth(100)

        # Query Count (read-only QLineEdit)
        self.query_count = QtWidgets.QLineEdit("0")
        self.query_count.setObjectName("-QUERY_COUNT-")
        self.query_count.setReadOnly(True)
        self.query_count.setFixedHeight(24)
        self.query_count.setMinimumWidth(50)

        # Query Title Frame with QLineEdit
        title_frame = QtWidgets.QFrame()
        title_layout = QtWidgets.QHBoxLayout(title_frame)
        title_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.query_title = QtWidgets.QLineEdit("title of prompt")
        self.query_title.setObjectName("-TITLE_INPUT-")
        self.query_title.setFixedHeight(24)
        self.query_title.setMinimumWidth(200)
        title_layout.addWidget(self.query_title)

        # Response Nav Frame: back, number, forward
        resp_nav_frame = QtWidgets.QFrame()
        resp_nav_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        resp_nav_layout = QtWidgets.QHBoxLayout(resp_nav_frame)
        self.resp_back_btn = QtWidgets.QPushButton("<-")
        self.resp_back_btn.setObjectName("-RESPONSE_TEXT_BACK-")
        self.resp_number = QtWidgets.QLineEdit("0")
        self.resp_number.setObjectName("-RESPONSE_TEXT_NUMBER-")
        self.resp_number.setFixedWidth(40)
        self.resp_forward_btn = QtWidgets.QPushButton("->")
        self.resp_forward_btn.setObjectName("-RESPONSE_TEXT_FORWARD-")
        self.resp_back_btn.setFixedHeight(24)
        self.resp_number.setFixedHeight(24)
        self.resp_forward_btn.setFixedHeight(24)
        resp_nav_layout.addWidget(self.resp_back_btn)
        resp_nav_layout.addWidget(self.resp_number)
        resp_nav_layout.addWidget(self.resp_forward_btn)

        # Assemble layouts
        left_layout = QtWidgets.QVBoxLayout()
        row1 = QtWidgets.QHBoxLayout()
        row1.addWidget(self.progress_text)
        row1.addWidget(self.progress_bar)
        row1.addWidget(self.query_count)
        left_layout.addLayout(row1)

        row2 = QtWidgets.QHBoxLayout()
        row2.addWidget(title_frame)
        row2.addWidget(resp_nav_frame)
        left_layout.addLayout(row2)

        self.setLayout(left_layout)


class OutputOptionsFrame(QtWidgets.QFrame):
    def __init__(self, parent=None):
        super().__init__(parent)
        layout = QtWidgets.QHBoxLayout(self)
        layout.setSpacing(20)

        self.btn_submit = QtWidgets.QPushButton("SUBMIT QUERY")
        self.btn_submit.setObjectName("-SUBMIT_QUERY-")
        self.btn_clear_requests = QtWidgets.QPushButton("CLEAR REQUESTS")
        self.btn_clear_requests.setObjectName("-CLEAR_REQUESTS-")
        self.btn_clear_chunks = QtWidgets.QPushButton("CLEAR CHUNKS")
        self.btn_clear_chunks.setObjectName("-CLEAR_CHUNKS-")
        self.btn_gen_readme = QtWidgets.QPushButton("GEN README")
        self.btn_gen_readme.setObjectName("-GENERATE_README-")

        for btn in (self.btn_submit, self.btn_clear_requests, self.btn_clear_chunks, self.btn_gen_readme):
            btn.setFixedHeight(32)
            btn.setMinimumWidth(120)
            layout.addWidget(btn)

        self.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.setLayout(layout)


class NavigationBar(QtWidgets.QWidget):
    def __init__(self, name: str, section: bool = True, parent=None):
        super().__init__(parent)
        nav_layout = QtWidgets.QHBoxLayout(self)
        nav_layout.setContentsMargins(0, 0, 0, 0)
        push_spacer1 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        nav_layout.addItem(push_spacer1)

        insert = f"{name} {'SECTION ' if section else ''}"
        self.back_btn = QtWidgets.QPushButton("<-")
        self.back_btn.setObjectName(f"-{insert.upper()}BACK-")
        self.number_edit = QtWidgets.QLineEdit("0")
        self.number_edit.setObjectName(f"-{insert.upper()}NUMBER-")
        self.number_edit.setFixedWidth(40)
        self.forward_btn = QtWidgets.QPushButton("->")
        self.forward_btn.setObjectName(f"-{insert.upper()}FORWARD-")
        for widget in (self.back_btn, self.number_edit, self.forward_btn):
            widget.setFixedHeight(24)
            nav_layout.addWidget(widget)

        push_spacer2 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        nav_layout.addItem(push_spacer2)
        self.setLayout(nav_layout)


class PromptTab(QtWidgets.QWidget):
    def __init__(self, title: str, with_subsection: bool = False, parent=None):
        super().__init__(parent)
        v_layout = QtWidgets.QVBoxLayout(self)
        # Section navigation
        v_layout.addWidget(NavigationBar(title, section=True))
        if with_subsection:
            v_layout.addWidget(NavigationBar(title, section=False))
        # Data frame for multiline text
        frame = QtWidgets.QGroupBox(f"{title} DATA")
        frame_layout = QtWidgets.QVBoxLayout(frame)
        self.text_edit = QtWidgets.QTextEdit()
        self.text_edit.setObjectName(f"-{title.upper()}-")
        frame_layout.addWidget(self.text_edit)
        v_layout.addWidget(frame)
        self.setLayout(v_layout)


class InstructionsTab(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        v_layout = QtWidgets.QVBoxLayout(self)
        # Section navigation
        v_layout.addWidget(NavigationBar("instructions", section=True))
        # Main instructions frame (with two sections: boolean text and sub_layout)
        frame = QtWidgets.QGroupBox("INSTRUCTIONS")
        frame_layout = QtWidgets.QVBoxLayout(frame)

        # Upper row: multiline for "instructions"
        self.instructions_edit = QtWidgets.QTextEdit()
        self.instructions_edit.setObjectName("-INSTRUCTIONS_TEXT-")
        self.instructions_edit.setPlaceholderText("instructions")
        frame_layout.addWidget(self.instructions_edit)

        # Lower row: other boolean instruction fields
        sub_layout_widget = QtWidgets.QWidget()
        sub_layout = QtWidgets.QGridLayout(sub_layout_widget)
        instruction_keys = [
            "additional_responses", "suggestions", "abort", "database_query",
            "notation", "generate_title", "additional_instruction", "request_chunks", "prompt_as_previous", "token_adjustment"
        ]
        self.checkbox_map = {}
        for idx, key in enumerate(instruction_keys):
            checkbox = QtWidgets.QCheckBox(key.replace('_', ' ').title())
            checkbox.setObjectName(f"-{key.upper()}_BOOL-")
            self.checkbox_map[key] = checkbox
            row = idx // 2
            col = idx % 2
            sub_layout.addWidget(checkbox, row, col)
        frame_layout.addWidget(sub_layout_widget)

        v_layout.addWidget(frame)
        self.setLayout(v_layout)


class PromptTabsWidget(QtWidgets.QTabWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        # Create and add tabs
        self.request_tab = PromptTab("request", with_subsection=False)
        self.addTab(self.request_tab, "REQUEST")
        self.prompt_data_tab = PromptTab("prompt data", with_subsection=False)
        self.addTab(self.prompt_data_tab, "PROMPT DATA")
        self.chunks_tab = PromptTab("chunks", with_subsection=True)
        self.addTab(self.chunks_tab, "CHUNKS")
        self.query_tab = PromptTab("query", with_subsection=True)
        self.addTab(self.query_tab, "QUERY")
        self.instructions_tab = InstructionsTab()
        self.addTab(self.instructions_tab, "INSTRUCTIONS")
        self.setObjectName("-PROMPT_TABS-")

class SettingsTab(QtWidgets.QScrollArea):
    def __init__(self, parent=None):
        super().__init__(parent)
        widget = QtWidgets.QWidget()
        grid = QtWidgets.QGridLayout(widget)

        # Token Percentage Frame
        token_frame = QtWidgets.QGroupBox("Token Percentage")
        token_layout = QtWidgets.QGridLayout(token_frame)
        token_keys = ['prompt percentage', 'completion percentage']
        self.token_combos = {}
        for i, key in enumerate(token_keys):
            label = QtWidgets.QLabel(key.title())
            combo = QtWidgets.QComboBox()
            combo.setObjectName(f"-{key.upper()}-")
            combo.addItems([str(i) for i in range(0, 101)])
            combo.setCurrentText("50")
            token_layout.addWidget(label, i, 0)
            token_layout.addWidget(combo, i, 1)
            self.token_combos[key] = combo
        grid.addWidget(token_frame, 0, 0, 1, 2)

        # API Options Frame
        api_frame = QtWidgets.QGroupBox("API Options")
        api_layout = QtWidgets.QFormLayout(api_frame)
        api_keys = ["header", "api key", "api env"]
        self.api_inputs = {}
        for key in api_keys:
            line_edit = QtWidgets.QLineEdit()
            line_edit.setObjectName(f"-{key.upper()}-")
            api_layout.addRow(key.title() + ":", line_edit)
            self.api_inputs[key] = line_edit
        grid.addWidget(api_frame, 1, 0, 1, 2)

        # Type Options Frame
        type_frame = QtWidgets.QGroupBox("Type Options")
        type_layout = QtWidgets.QFormLayout(type_frame)
        self.role_combo = QtWidgets.QComboBox()
        self.role_combo.setObjectName("-ROLE-")
        roles = ['assistant', 'Elaborative', 'Socratic', 'Concise', 'Friendly/Conversational',
                 'Professional/Formal', 'Role-Playing', 'Teaching', "Debative/Devil's Advocate",
                 'Creative/Brainstorming', 'Empathetic/Supportive']
        self.role_combo.addItems(roles)
        type_layout.addRow("Role:", self.role_combo)
        self.resp_type_combo = QtWidgets.QComboBox()
        self.resp_type_combo.setObjectName("-RESPONSE_TYPE-")
        self.resp_type_combo.addItems(['instruction', 'json', 'bash', 'text'])
        type_layout.addRow("Response Type:", self.resp_type_combo)
        grid.addWidget(type_frame, 2, 0, 1, 2)

        # Prompt Options Frame (just one checkbox)
        prompt_frame = QtWidgets.QGroupBox("Prompt Options")
        prompt_layout = QtWidgets.QHBoxLayout(prompt_frame)
        self.prompt_as_received_cb = QtWidgets.QCheckBox("Prompt As Received")
        self.prompt_as_received_cb.setObjectName("-PROMPT_AS_RECEIVED-")
        self.prompt_as_received_cb.setChecked(True)
        prompt_layout.addWidget(self.prompt_as_received_cb)
        grid.addWidget(prompt_frame, 3, 0, 1, 2)

        # Tokens Frame (display only)
        tokens_frame = QtWidgets.QGroupBox("Tokens")
        tokens_layout = QtWidgets.QVBoxLayout(tokens_frame)
        token_display_keys = ['completion tokens available', 'completion tokens desired', 'completion tokens used']
        self.token_displays = {}
        for label_text in token_display_keys:
            sub_frame = QtWidgets.QFrame()
            sub_layout = QtWidgets.QHBoxLayout(sub_frame)
            label = QtWidgets.QLabel(label_text.title())
            line_edit = QtWidgets.QLineEdit("0")
            line_edit.setObjectName(f"-{label_text.upper()}-")
            line_edit.setReadOnly(True)
            line_edit.setFixedWidth(60)
            sub_layout.addWidget(label)
            sub_layout.addWidget(line_edit)
            tokens_layout.addWidget(sub_frame)
            self.token_displays[label_text] = line_edit
        grid.addWidget(tokens_frame, 4, 0, 1, 2)

        # Title Frame
        title_frame = QtWidgets.QGroupBox("Title")
        title_layout = QtWidgets.QHBoxLayout(title_frame)
        self.title_bool_cb = QtWidgets.QCheckBox("Title")
        self.title_bool_cb.setObjectName("-TITLE_BOOL-")
        self.title_bool_cb.setChecked(True)
        self.title_input = QtWidgets.QLineEdit()
        self.title_input.setObjectName("-TITLE_TEXT-")
        title_layout.addWidget(self.title_bool_cb)
        title_layout.addWidget(self.title_input)
        grid.addWidget(title_frame, 5, 0, 1, 2)

        # Model Select Frame
        model_frame = QtWidgets.QGroupBox("Model Select")
        model_layout = QtWidgets.QFormLayout(model_frame)
        self.endpoint_input = QtWidgets.QLineEdit()
        self.endpoint_input.setObjectName("-ENDPOINT-")
        self.endpoint_input.setReadOnly(True)
        model_layout.addRow("Endpoint:", self.endpoint_input)
        self.model_combo = QtWidgets.QComboBox()
        self.model_combo.setObjectName("-MODEL-")
        # Placeholder items; in practice populate from ModelManager
        self.model_combo.addItems(["model1", "model2", "model3"])
        model_layout.addRow("Model:", self.model_combo)
        self.max_tokens_display = QtWidgets.QLineEdit("8200")
        self.max_tokens_display.setObjectName("-MAX_TOKENS-")
        self.max_tokens_display.setReadOnly(True)
        model_layout.addRow("Tokens:", self.max_tokens_display)
        grid.addWidget(model_frame, 6, 0, 1, 2)

        # Enable Instructions Frame (checkboxes grid)
        instr_frame = QtWidgets.QGroupBox("Enable Instructions")
        instr_layout = QtWidgets.QGridLayout(instr_frame)
        instruction_keys = ["instructions", "additional_responses", "suggestions", "abort",
                            "database_query", "notation", "generate_title", "additional_instruction", "request_chunks", "prompt_as_previous", "token_adjustment"]
        self.instr_checkboxes = {}
        for idx, key in enumerate(instruction_keys):
            cb = QtWidgets.QCheckBox(key.replace('_', ' ').title())
            cb.setObjectName(f"-{key.upper()}_BOOL-")
            if key in ['instructions', 'generate_title', 'suggestions']:
                cb.setChecked(True)
            row = idx // 4
            col = idx % 4
            instr_layout.addWidget(cb, row, col)
            self.instr_checkboxes[key] = cb
        grid.addWidget(instr_frame, 7, 0, 1, 2)

        # Test Tools Frame
        test_frame = QtWidgets.QGroupBox("Test Tools")
        test_layout = QtWidgets.QHBoxLayout(test_frame)
        self.test_run_cb = QtWidgets.QCheckBox("test run")
        self.test_run_cb.setObjectName("-TEST_RUN-")
        self.test_files_cb = QtWidgets.QCheckBox("test files")
        self.test_files_cb.setObjectName("-TEST_FILES-")
        self.test_file_input = QtWidgets.QLineEdit()
        self.test_file_input.setObjectName("-TEST_FILE_INPUT-")
        self.test_browse_btn = QtWidgets.QPushButton("Browse")
        self.test_browse_btn.setObjectName("-TEST_BROWSE-")
        test_layout.addWidget(self.test_run_cb)
        test_layout.addWidget(self.test_files_cb)
        test_layout.addWidget(self.test_file_input)
        test_layout.addWidget(self.test_browse_btn)
        grid.addWidget(test_frame, 8, 0, 1, 2)

        # File Options Frame
        file_frame = QtWidgets.QGroupBox("File Options")
        file_layout = QtWidgets.QVBoxLayout(file_frame)
        self.file_options_cbs = {}
        for key in ['auto chunk title', 'reuse chunk data', 'append chunks', 'scan mode all']:
            cb = QtWidgets.QCheckBox(key.replace('_', ' ').title())
            cb.setObjectName(f"-{key.upper()}-")
            file_layout.addWidget(cb)
            self.file_options_cbs[key] = cb
        grid.addWidget(file_frame, 9, 0, 1, 2)

        widget.setLayout(grid)
        self.setWidget(widget)
        self.setWidgetResizable(True)


class ResponsesTab(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        v_layout = QtWidgets.QVBoxLayout(self)
        # Collate Responses & JSON to String checkboxes
        h_layout = QtWidgets.QHBoxLayout()
        self.collate_cb = QtWidgets.QCheckBox("Collate Responses")
        self.collate_cb.setObjectName("-COLLATE_RESPONSES_BOOL-")
        self.json_to_string_cb = QtWidgets.QCheckBox("JSON to String")
        self.json_to_string_cb.setObjectName("-FORMAT_JSON_TO_STRING_RESPONSES-")
        h_layout.addWidget(self.collate_cb)
        h_layout.addWidget(self.json_to_string_cb)

        # Response Key Combo Box
        self.response_key_combo = QtWidgets.QComboBox()
        self.response_key_combo.setObjectName("-RESPONSE_KEY_SELECTION_RESPONSES-")
        key_frame = QtWidgets.QGroupBox("Response Key")
        key_layout = QtWidgets.QHBoxLayout(key_frame)
        key_layout.addWidget(self.response_key_combo)

        v_layout.addLayout(h_layout)
        v_layout.addWidget(key_frame)

        # File text multiline
        self.file_text_edit = QtWidgets.QTextEdit()
        self.file_text_edit.setObjectName("-FILE_TEXT_RESPONSES-")
        self.file_text_edit.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        v_layout.addWidget(self.file_text_edit)

        self.setLayout(v_layout)


class FilesTab(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        v_layout = QtWidgets.QVBoxLayout(self)

        # Chunk title input
        title_layout = QtWidgets.QHBoxLayout()
        self.chunk_title_input = QtWidgets.QLineEdit()
        self.chunk_title_input.setObjectName("-CHUNK_TITLE_FILES-")
        title_layout.addWidget(QtWidgets.QLabel("Chunk Title:"))
        title_layout.addWidget(self.chunk_title_input)
        v_layout.addLayout(title_layout)

        # Buttons: CHUNK_DATA, RESPONSE_DATA
        btn_layout = QtWidgets.QHBoxLayout()
        self.chunk_data_btn = QtWidgets.QPushButton("CHUNK_DATA")
        self.chunk_data_btn.setObjectName("-ADD_FILE_TO_CHUNK_FILES-")
        self.response_data_btn = QtWidgets.QPushButton("RESPONSE_DATA")
        self.response_data_btn.setObjectName("-ADD_FILE_TO_RESPONSE_FILES-")
        btn_layout.addWidget(self.chunk_data_btn)
        btn_layout.addWidget(self.response_data_btn)
        v_layout.addLayout(btn_layout)

        # File text multiline
        self.file_text_edit = QtWidgets.QTextEdit()
        self.file_text_edit.setObjectName("-FILE_TEXT_FILES-")
        self.file_text_edit.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        v_layout.addWidget(self.file_text_edit)

        self.setLayout(v_layout)


class QueryTab(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        v_layout = QtWidgets.QVBoxLayout(self)

        # Database Query & Perform Query checkboxes
        h_layout = QtWidgets.QHBoxLayout()
        self.database_query_cb = QtWidgets.QCheckBox("Database Query")
        self.database_query_cb.setObjectName("-DATABASE_QUERY_BOOL_DATABASE-")
        self.perform_query_cb = QtWidgets.QCheckBox("Perform Query")
        self.perform_query_cb.setObjectName("-PERFORM_QUERY_BOOL_DATABASE-")
        h_layout.addWidget(self.database_query_cb)
        h_layout.addWidget(self.perform_query_cb)
        v_layout.addLayout(h_layout)

        # Table configuration combo
        table_frame = QtWidgets.QFrame()
        table_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        table_layout = QtWidgets.QHBoxLayout(table_frame)
        self.table_combo = QtWidgets.QComboBox()
        self.table_combo.setObjectName("-CHUNK_TITLE_DATABASE-")
        # Placeholder: populate with actual table names
        self.table_combo.addItems(["table1", "table2", "table3"])
        table_layout.addWidget(QtWidgets.QLabel("Table:"))
        table_layout.addWidget(self.table_combo)
        v_layout.addWidget(table_frame)

        # Buttons: CHUNK_DATA, RESPONSE_DATA
        btn_layout = QtWidgets.QHBoxLayout()
        self.chunk_data_btn = QtWidgets.QPushButton("CHUNK_DATA")
        self.chunk_data_btn.setObjectName("-ADD_FILE_TO_CHUNK_DATABASE-")
        self.response_data_btn = QtWidgets.QPushButton("RESPONSE_DATA")
        self.response_data_btn.setObjectName("-ADD_FILE_TO_RESPONSE_DATABASE-")
        btn_layout.addWidget(self.chunk_data_btn)
        btn_layout.addWidget(self.response_data_btn)
        v_layout.addLayout(btn_layout)

        # File text multiline
        self.file_text_edit = QtWidgets.QTextEdit()
        self.file_text_edit.setObjectName("-FILE_TEXT_DATABASE-")
        self.file_text_edit.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        v_layout.addWidget(self.file_text_edit)

        self.setLayout(v_layout)


class UrlsTab(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        v_layout = QtWidgets.QVBoxLayout(self)

        # URL input and button
        url_input_layout = QtWidgets.QHBoxLayout()
        self.url_input = QtWidgets.QLineEdit()
        self.url_input.setObjectName("-URL-")
        self.add_url_btn = QtWidgets.QPushButton("Add URL")
        self.add_url_btn.setObjectName("-ADD_URL-")
        url_input_layout.addWidget(self.url_input)
        url_input_layout.addWidget(self.add_url_btn)
        v_layout.addLayout(url_input_layout)

        # URL listbox
        self.url_listbox = QtWidgets.QListWidget()
        self.url_listbox.setObjectName("-URL_LIST-")
        v_layout.addWidget(self.url_listbox)

        # GET SOUP, GET SOURCE, CHUNK_DATA buttons
        btn_layout = QtWidgets.QHBoxLayout()
        self.get_soup_btn = QtWidgets.QPushButton("GET SOUP")
        self.get_soup_btn.setObjectName("-GET_SOUP-")
        self.get_source_btn = QtWidgets.QPushButton("GET SOURCE")
        self.get_source_btn.setObjectName("-GET_SOURCE_CODE-")
        self.chunk_data_btn = QtWidgets.QPushButton("CHUNK_DATA")
        self.chunk_data_btn.setObjectName("-ADD_URL_TO_CHUNK-")
        btn_layout.addWidget(self.get_soup_btn)
        btn_layout.addWidget(self.get_source_btn)
        btn_layout.addWidget(self.chunk_data_btn)
        v_layout.addLayout(btn_layout)

        # Chunk title input
        chunk_title_frame = QtWidgets.QFrame()
        chunk_title_layout = QtWidgets.QHBoxLayout(chunk_title_frame)
        self.chunk_title_input = QtWidgets.QLineEdit()
        self.chunk_title_input.setObjectName("-CHUNK_TITLE_URL-")
        chunk_title_layout.addWidget(QtWidgets.QLabel("Chunk Title:"))
        chunk_title_layout.addWidget(self.chunk_title_input)
        v_layout.addWidget(chunk_title_frame)

        # URL text multiline
        self.url_text_edit = QtWidgets.QTextEdit()
        self.url_text_edit.setObjectName("-URL_TEXT-")
        self.url_text_edit.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        v_layout.addWidget(self.url_text_edit)

        self.setLayout(v_layout)


class FeedbackTab(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        main_layout = QtWidgets.QVBoxLayout(self)

        # Response Frame
        response_frame = QtWidgets.QGroupBox("Response")
        response_layout = QtWidgets.QVBoxLayout(response_frame)
        self.response_edit = QtWidgets.QTextEdit()
        self.response_edit.setObjectName("-RESPONSE-")
        self.response_edit.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        response_layout.addWidget(self.response_edit)
        main_layout.addWidget(response_frame)

        # Other feedback fields
        fields = ["request_chunks", "abort", "additional_responses", "suggestions", "notation", "other"]
        self.feedback_widgets = {}
        for field in fields:
            frame = QtWidgets.QGroupBox(field.replace('_', ' ').title())
            layout = QtWidgets.QVBoxLayout(frame)
            if field in ["request_chunks", "abort", "additional_responses"]:
                line = QtWidgets.QLineEdit()
                line.setObjectName(f"-{field.upper()}_FEEDBACK-")
                layout.addWidget(line)
                self.feedback_widgets[field] = line
            else:
                multiline = QtWidgets.QTextEdit()
                multiline.setObjectName(f"-{field.upper()}_FEEDBACK-")
                multiline.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
                layout.addWidget(multiline)
                self.feedback_widgets[field] = multiline
            main_layout.addWidget(frame)

        self.setLayout(main_layout)


class UtilitiesTabsWidget(QtWidgets.QTabWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.settings_tab = SettingsTab()
        self.addTab(self.settings_tab, "SETTINGS")
        self.responses_tab = ResponsesTab()
        self.addTab(self.responses_tab, "RESPONSES")
        self.files_tab = FilesTab()
        self.addTab(self.files_tab, "FILES")
        self.query_tab = QueryTab()
        self.addTab(self.query_tab, "QUERY")
        self.urls_tab = UrlsTab()
        self.addTab(self.urls_tab, "URLS")
        self.feedback_tab = FeedbackTab()
        self.addTab(self.feedback_tab, "FEEDBACK")
        self.setObjectName("-UTILITIES_TABS-")


class MainAiGui(QtWidgets.QMainWindow):
    def __init__(self, gpt_manager):
        super().__init__()
        self.gpt_manager = gpt_manager  # Reference to GptManager for event handling
        self.setWindowTitle("ChatGPT Console")
        self.resize(1200, 800)

        central_widget = QtWidgets.QWidget()
        main_layout = QtWidgets.QVBoxLayout(central_widget)
        main_layout.setSpacing(20)

        # 1) Progress Frame
        self.progress_frame = ProgressFrame()
        main_layout.addWidget(self.progress_frame)

        # 2) Output Options
        self.output_options = OutputOptionsFrame()
        main_layout.addWidget(self.output_options)

        # 3) Main horizontal split: Prompt Tabs (left) + Utilities Tabs (right)
        splitter = QtWidgets.QSplitter(QtCore.Qt.Horizontal)

        # Prompt Tabs
        self.prompt_tabs = PromptTabsWidget()
        splitter.addWidget(self.prompt_tabs)

        # Utilities Tabs
        self.utilities_tabs = UtilitiesTabsWidget()
        splitter.addWidget(self.utilities_tabs)

        splitter.setStretchFactor(0, 1)
        splitter.setStretchFactor(1, 2)
        main_layout.addWidget(splitter)

        self.setCentralWidget(central_widget)

        # Optional: status bar
        self.status_bar = self.statusBar()
        self.status_bar.showMessage("Ready")

        # Connect signals to GptManager's on_event
        self._connect_signals()
def main():
    import sys
    app = QtWidgets.QApplication(sys.argv)
    window = MainAiGui(GptManager)
    window.show()
    sys.exit(app.exec_())


main()



#!/usr/bin/env python3
# gui_frontend.py (with a toggleable log console)
from __future__ import annotations

# ─── Standard library ─────────────────────────────────────────────────────────
import ast,datetime,json,os,glob,re,sys,traceback
from pathlib import Path

from typing import *

# ─── Third-party ──────────────────────────────────────────────────────────────
from PyQt5 import(QtCore,
                  QtGui,
                  QtWidgets,
                  QtWebChannel,
                  QtWebEngineWidgets
                  )
import pytesseract
from pdf2image import convert_from_path

# ─── Local application ─────────────────────────────────────────────────────────
from abstract_pandas import get_df
from abstract_utilities import (SingletonMeta,
                                get_logFile,
                                read_from_file,
                                make_list
                                )
from abstract_utilities.robust_reader import (
    collect_filepaths,
    DEFAULT_EXCLUDE_DIRS,
    DEFAULT_EXCLUDE_FILE_PATTERNS,
    read_file_as_text,
)

from PyQt5 import(QtCore,
                  QtGui,
                  QtWidgets,
                  QtWebChannel,
                  QtWebEngineWidgets
                  )



from ...imports import *
def _clear_layout(layout: QtWidgets.QLayout):
    """Recursively delete all widgets in a layout (Qt-safe)."""
    if layout is None:
        return
    while layout.count():
        item = layout.takeAt(0)
        if item.widget():
            item.widget().setParent(None)      # detach
            item.widget().deleteLater()        # mark for deletion
        elif item.layout():
            _clear_layout(item.layout())
def _rebuild_ext_row(self, paths: List[str]):
    """
    1. Collect unique extensions from `paths` (only actual files).
    2. Wipe any existing checkboxes.
    3. Create a checkbox for each extension, preserving prior states.
    4. Show the row, or hide if no extensions found.
    """
    exts = {os.path.splitext(p)[1].lower() for p in paths if os.path.isfile(p)}
    exts.discard("")   # drop empty ext (e.g. directories)
    self._log(f"Unique extensions: {exts}")

    if not exts:
        # No file‐extensions → hide the row
        self.ext_row.setVisible(False)
        self.ext_checks.clear()
        return

    # Remove any existing checkboxes from the layout
    _clear_layout(self.ext_row_lay)

    new_checks: dict[str, QtWidgets.QCheckBox] = {}
    for ext in sorted(exts):
        cb = QtWidgets.QCheckBox(ext)
        # Keep prior check state if exists, else default True
        cb.setChecked(self.ext_checks.get(ext, True))
        cb.stateChanged.connect(self._apply_ext_filter)
        self.ext_row_lay.addWidget(cb)
        new_checks[ext] = cb

    # Replace with new map
    self.ext_checks = new_checks
    # Make the row visible now that it has content
    self.ext_row.setVisible(True)



#!/usr/bin/env python3
# main.py

import os
os.environ["QT_XCB_GL_INTEGRATION"] = "none"

from PyQt5 import QtCore, QtWidgets
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseSoftwareOpenGL)
from .drag_and_drop import DragDropWithFileBrowser, DragDropWithWebBrowser
from .file_drop_area  import FileDropArea
from .file_tree       import FileSystemTree
from .JSBridge import JSBridge
from .imports import *
import sys

def gui_main():
    win = DragDropWithFileBrowser
    run_app(win=win,win_kwargs={"FileSystemTree":FileSystemTree,"FileDropArea":FileDropArea})


def gui_web(target_url=None):
    from .JSBridge import JSBridge
    app = QtWidgets.QApplication(sys.argv)

    # Pass the URL into the constructor of your widget
    window = DragDropWithWebBrowser(
        FileDropArea=FileDropArea,
        FileSystemTree=FileSystemTree,
        JSBridge=JSBridge,
        url=target_url,                 # new parameter
    )
    window.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1].lower() == "web":
        # e.g. `python -m clipit.main web https://my.site/to/inspect`
        url_to_load = sys.argv[2] if len(sys.argv) > 2 else None
        gui_web(target_url=url_to_load)
    else:
        gui_main()



from abstract_utilities import safe_json_loads,make_list,os,get_any_value
from .responseContentParser import get_updated_response_content,get_file_path,get_title
from abstract_database import ensure_db_manager,flatten_json,pd
from abstract_apis import get_async_response
from abstract_pandas import safe_excel_save
import asyncio,logging
from concurrent.futures import ThreadPoolExecutor
from abstract_database import *
logging.basicConfig(
    level=logging.INFO,                     # Set the logging level (INFO, in this case)
    format='%(asctime)s - %(levelname)s - %(message)s',  # Log format
    handlers=[
        logging.StreamHandler()             # Also log to the console (optional)
    ]
)
logger = logging.getLogger(__name__)
# Define the executor globally
executor = ThreadPoolExecutor()
def extract_query(database_query):
    # Check if 'query' key exists, otherwise fallback to other keys
    if 'query' in database_query:
        return database_query['query']
    elif 'searchQuery' in database_query:
        return database_query['searchQuery']
    elif isinstance(database_query, dict):
        # Attempt to find any valid query-like string in the dictionary
        for key, value in database_query.items():
            if 'select' in value.lower() or 'query' in key.lower():
                return value
    return None

from ..gpt_classes.nogui_selection.general_query import make_general_query
async def async_safe_excel_save(df, original_file_path, index=False, suffix=None, engine=None):
    """
    Asynchronous wrapper for safe_excel_save, running it in an executor.
    """
    loop = asyncio.get_event_loop()
    return await loop.run_in_executor(
        executor, safe_excel_save, df, original_file_path, index, suffix, engine
    )
async def save_to_excel_async(rows, file_path="output.xlsx"):
    """Save query results to an Excel file asynchronously."""
    excel_data = []
    if rows:
        for row in rows:
            # Ensure the row is a list before flattening
            row = list(row) if isinstance(row, tuple) else row
            # Flatten JSON and append to data
            excel_data.append(flatten_json(row, parent_key='', sep='_'))

        # Convert data to a pandas DataFrame
        df = pd.DataFrame(excel_data)

        # Use the asynchronous safe save function
        await async_safe_excel_save(df, file_path)
        logging.info(f"Excel file saved asynchronously to {file_path}.")
def search_multiple(query,conn):

    cur = conn.cursor()
    try:
        cur.execute(**query)
        results = cur.fetchall()
        return results
    except psycopg2.Error as e:
        print(f"Error querying JSONB data: {e}")
    finally:
        cur.close()
        conn.close()
async def search_multiple_fields(query_result=None,db_mgr=None,env_path=None,dbType=None,dbName=None,conn_mgr=None):
    db_mgr = ensure_db_manager(db_mgr=db_mgr,env_path=env_path,dbType=dbType,dbName=dbName,conn_mgr=conn_mgr)
    message = f"query successful"
    error=False
    try:
        response = search_multiple(query_result,conn_mgr.connect_db())
    except Exception as e:
        response = None
        message=f"the query with query_result: {query_result} errored with: {e}"
        logging.error(message)
        error=True
    return {"error":error,"message":message,"response":response}
async def async_get_xlsx_file_path_from_response_vars(data=None,file_path=None,db_mgr=None,env_path=None,dbType=None,dbName=None,conn_mgr=None):
    db_mgr = ensure_db_manager(db_mgr=db_mgr,env_path=env_path,dbType=dbType,dbName=dbName,conn_mgr=conn_mgr)
    response_content = get_updated_response_content(data=data,file_path=file_path)
    new_directory = os.path.join(os.path.dirname(get_file_path(response_content)), 'queries')
    os.makedirs(new_directory, exist_ok=True)
    new_file_path = os.path.join(new_directory, f"{get_title(response_content)}.xlsx")
    return new_file_path
async def async_get_raw_response_for_query(prompt,tableName,db_mgr=None,env_path=None,dbType=None,dbName=None,conn_mgr=None):
    db_mgr = ensure_db_manager(db_mgr=db_mgr,env_path=env_path,dbType=dbType,dbName=dbName,conn_mgr=conn_mgr)
    data = db_mgr.get_instruction_from_tableName(tableName)
    raw_response = await make_general_query(prompt=[prompt],env_path=env_path,data=str(data),instruction_bools={"database_query":True,"suggestions":True})
    if isinstance(raw_response,list):
        raw_response = safe_json_loads(raw_response[0])
    return raw_response
async def async_get_auto_db_query(prompt, tableName, db_mgr=None, env_path=None, dbType=None, dbName=None, conn_mgr=None):
    db_mgr = ensure_db_manager(db_mgr=db_mgr, env_path=env_path, dbType=dbType, dbName=dbName, conn_mgr=conn_mgr)
    
    # Get the raw AI response
    raw_response = await async_get_raw_response_for_query(prompt, tableName, db_mgr=db_mgr)
  
    # Update the response content
    response_content = get_updated_response_content(data=raw_response)
    
    # Ensure we have an Excel file path to save the results
    xlsx_file_path = await async_get_xlsx_file_path_from_response_vars(data=raw_response)
    response_content["excelPath"] = xlsx_file_path
    
    # Extract the query from the database_query field
    database_query = response_content.get('database_query', {})
    query_result = extract_query(database_query)
    
    if not query_result:
        raise ValueError("Unable to extract query from response")
    
    # Run the query using the extracted query_result
    query_response = await search_multiple_fields(query_result={"query": query_result}, db_mgr=db_mgr, env_path=env_path, dbType=dbType, dbName=dbName, conn_mgr=conn_mgr)
    # If query was successful, save the results to an Excel file
    if query_response.get('error') in [False,None] :
        await save_to_excel_async(rows=query_response.get('response'), file_path=xlsx_file_path)
    
    # Update the response with query response
    response_content.update(query_response)
    return response_content
def get_auto_db_query(prompt,tableName,db_mgr=None,env_path=None,dbType=None,dbName=None,conn_mgr=None):
    db_mgr = ensure_db_manager(db_mgr=db_mgr, env_path=env_path, dbType=dbType, dbName=dbName, conn_mgr=conn_mgr)
    if conn_mgr == None:
        conn_mgr = db_mgr.conn_mgr
    prompt+='''\n\n###EXTRA DATABASE INSTRUCTION###\n\nt. the time is '''+f"{time.time()}\n"+'''Always include a 'query' key containing the SQL query as a string. Example: {\"query\": \"SELECT * FROM table WHERE condition\"}. Ensure that the 'query' key is always present in the database_query object, and the query is valid SQL.\n\n"'''
    return get_async_response(async_get_auto_db_query,prompt,tableName,db_mgr=db_mgr,env_path=env_path,dbType=dbType,dbName=dbName,conn_mgr=conn_mgr)



import inspect
from PyQt5 import QtWidgets, QtCore

# 1) Constructor signature(s)
print(inspect.signature(QtWidgets.QListWidgetItem.__init__))
# e.g. __init__(self, *args, **kwargs)  # overloaded under the hood

# 2) All public members (methods, signals, properties, …)
all_members = [m for m in dir(QtWidgets.QListWidgetItem) if not m.startswith('_')]
print("All members:", all_members)

# 3) Which of those are callables?
methods = [m for m in all_members
           if callable(getattr(QtWidgets.QListWidgetItem, m))]
print("Methods & signals:", methods)

# 4) Which are non-callable properties?
props = [m for m in all_members
         if not callable(getattr(QtWidgets.QListWidgetItem, m))]
print("Attributes/properties:", props)

# 5) If you want to know what “roles” you can pass to item.data()/setData():

roles = [(name, getattr(QtCore.Qt, name))
         for name in dir(QtCore.Qt)
         if name.endswith('Role')]
print("Available data‐roles (name, int):\n", roles)



import sys
import os
from PyQt6.QtWidgets import (
    QApplication, QMainWindow, QWidget, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit,
    QProgressBar, QGroupBox, QPushButton, QCheckBox, QComboBox, QTextEdit, QTabWidget,
    QListWidget, QFileDialog, QTreeWidget, QTreeWidgetItem, QScrollArea, QMessageBox
)
from PyQt6.QtCore import Qt, QSize
from PyQt6.QtGui import QFont, QPixmap, QImage
from io import BytesIO
import requests
from PIL import Image

# Utility functions adapted from original
def ensure_nested_list(obj):
    if not isinstance(obj, list):
        return [obj]
    return obj

def expandable(size=(None, None), scroll_vertical=False, scroll_horizontal=False):
    # In PyQt, we can use size policies or fixed sizes; return dict for params
    params = {}
    if size[0]:
        params['fixed_width'] = size[0]
    if size[1]:
        params['fixed_height'] = size[1]
    return params

def text_to_key(text, section=''):
    return f"-{text.upper().replace(' ', '_')}{f'_{section.upper()}' if section else ''}-"

# Mock classes from original (simplified)
class AbstractWindowManager:
    def __init__(self):
        pass

class AbstractBrowser:
    def get_scan_browser_layout(self, section='', extra_buttons=[]):
        # Simplified browser layout: file browser with list and buttons
        layout_widget = QWidget()
        v_layout = QVBoxLayout()
        layout_widget.setLayout(v_layout)

        file_list = QListWidget()
        file_list.setObjectName(text_to_key('file list', section=section))
        v_layout.addWidget(file_list)

        browse_btn = QPushButton("Browse")
        browse_btn.clicked.connect(lambda: self.browse_files(file_list))
        v_layout.addWidget(browse_btn)

        for btn in extra_buttons:
            v_layout.addWidget(btn)

        return [layout_widget]  # Return as list for consistency

    def browse_files(self, list_widget):
        files = QFileDialog.getOpenFileNames()[0]
        for file in files:
            list_widget.addItem(file)

class RightClickManager:
    def get_right_click(self, key=''):
        # Context menu simulation; return None as PyQt handles via setContextMenuPolicy
        return None

right_click_mgr = RightClickManager()

# Widget creation function (replacement for make_component)
def make_widget(widget_type, parent=None, **kwargs):
    widget = None
    if widget_type == "Button":
        widget = QPushButton(kwargs.get("button_text", "Button"), parent)
        if 'disabled' in kwargs:
            widget.setEnabled(not kwargs['disabled'])
    elif widget_type == "Input":
        widget = QLineEdit(kwargs.get("default_text", ""), parent)
        if 'disabled' in kwargs:
            widget.setReadOnly(kwargs['disabled'])
        if 'size' in kwargs:
            w, h = kwargs['size']
            widget.setFixedSize(w * 10, h * 20)  # Approximate
    elif widget_type == "InputText":
        return make_widget("Input", parent, **kwargs)
    elif widget_type == "ProgressBar":
        widget = QProgressBar(parent)
        widget.setMaximum(100)
        if 'size' in kwargs:
            w, h = kwargs['size']
            widget.setFixedSize(w * 10, h * 20)
    elif widget_type == "Frame":
        widget = QGroupBox(kwargs.get("title", ""), parent)
        inner_layout = QVBoxLayout()
        widget.setLayout(inner_layout)
        if 'layout' in kwargs:
            build_layout(kwargs['layout'], inner_layout)
    elif widget_type == "Checkbox":
        widget = QCheckBox(kwargs.get("title", ""), parent)
        if 'default' in kwargs:
            widget.setChecked(kwargs['default'])
    elif widget_type == "Combo":
        widget = QComboBox(parent)
        if 'values' in kwargs:
            widget.addItems([str(v) for v in kwargs['values']])
        if 'default_value' in kwargs:
            widget.setCurrentText(str(kwargs['default_value']))
        if 'size' in kwargs:
            w, h = kwargs['size']
            widget.setFixedSize(w * 10, h * 20)
    elif widget_type == "Multiline":
        widget = QTextEdit(parent)
        if 'size' in kwargs:
            w, h = kwargs['size']
            widget.setFixedSize(w * 10, h * 20)
        # Simulate right-click if needed
        widget.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
    elif widget_type == "TabGroup":
        widget = QTabWidget(parent)
        if 'layout' in kwargs:
            for tab in ensure_nested_list(kwargs['layout']):
                tab_widget = QWidget()
                tab_layout = QVBoxLayout()
                tab_widget.setLayout(tab_layout)
                build_layout(tab.get('layout', []), tab_layout)
                widget.addTab(tab_widget, tab.get('title', 'Tab'))
    elif widget_type == "Tab":
        # Return dict for TabGroup
        return {'title': kwargs.get("title", "").upper(), 'layout': ensure_nested_list(kwargs.get("layout", []))}
    elif widget_type == "Column":
        widget = QWidget(parent)
        col_layout = QVBoxLayout()
        widget.setLayout(col_layout)
        if 'layout' in kwargs:
            build_layout(kwargs['layout'], col_layout)
    elif widget_type == "Text":
        widget = QLabel(kwargs.get("title", ""), parent)
    elif widget_type == "Listbox":
        widget = QListWidget(parent)
        if 'values' in kwargs:
            for v in kwargs['values']:
                widget.addItem(str(v))
        if 'size' in kwargs:
            w, h = kwargs['size']
            widget.setFixedSize(w * 10, h * 20)
    elif widget_type == "Tree":
        widget = QTreeWidget(parent)
        if 'headings' in kwargs:
            widget.setHeaderLabels(kwargs['headings'])
    else:
        raise ValueError(f"Unknown widget type: {widget_type}")

    if 'key' in kwargs:
        widget.setObjectName(kwargs['key'])

    # Apply expandable params
    if 'expandable' in kwargs:
        params = kwargs['expandable']
        if 'fixed_width' in params:
            widget.setFixedWidth(params['fixed_width'])
        if 'fixed_height' in params:
            widget.setFixedHeight(params['fixed_height'])

    return widget

# Build layout from nested list
def build_layout(layout_list, parent_layout):
    for row in ensure_nested_list(layout_list):
        row_layout = QHBoxLayout()
        for element in ensure_nested_list(row):
            if isinstance(element, list):
                col_widget = QWidget()
                col_layout = QVBoxLayout()
                col_widget.setLayout(col_layout)
                build_layout(element, col_layout)
                row_layout.addWidget(col_widget)
            else:
                row_layout.addWidget(element)
        parent_layout.addLayout(row_layout)

# Adapted functions from gui_utils.py and others
def get_standard_screen_dimensions(width=0.70, height=0.80):
    screen = QApplication.primaryScreen()
    rect = screen.availableGeometry()
    return int(rect.width() * width), int(rect.height() * height)

window_width, window_height = get_standard_screen_dimensions()

def get_left_right_nav(name, section=True, push=True):
    insert = f"{name} {'section ' if section else ''}"
    back_btn = make_widget("Button", button_text="<-", key=text_to_key(f"{insert}back"))
    number_input = make_widget("Input", default_text='0', key=text_to_key(f"{insert}number"), size=(4,1))
    forward_btn = make_widget("Button", button_text="->", key=text_to_key(f"{insert}forward"))

    nav_widget = QWidget()
    nav_layout = QHBoxLayout()
    nav_widget.setLayout(nav_layout)

    if push:
        nav_layout.addStretch()

    nav_layout.addWidget(back_btn)
    nav_layout.addWidget(number_input)
    nav_layout.addWidget(forward_btn)

    if push:
        nav_layout.addStretch()

    return nav_widget

def generate_bool_text(title, args={}):
    frame = make_widget("Frame", title=title)
    inner_layout = frame.layout()
    multi = make_widget("Multiline", key=text_to_key(title, section='text'), **args)
    inner_layout.addWidget(multi)
    return frame

def get_tab_layout(title, layout=None):
    if not layout:
        layout = make_widget("Multiline", key=text_to_key(title), **expandable(size=(None, 5)))
    tab_dict = make_widget("Tab", title=title, layout=layout)
    return tab_dict

def get_column(layout, args={}):
    col = make_widget("Column", layout=layout, **args)
    return col

def get_tab_group(grouped_tabs, args={}):
    tab_group = make_widget("TabGroup", layout=grouped_tabs, **args)
    return tab_group

def get_right_click_multi(key, args={}):
    multi = make_widget("Multiline", key=key, **args)
    return multi

# From progress_frame.py
def get_progress_frame():
    progress_widget = QWidget()
    h_layout = QHBoxLayout()
    progress_widget.setLayout(h_layout)

    # PROGRESS frame
    progress_group = make_widget("Frame", title='PROGRESS')
    progress_inner = QHBoxLayout()
    progress_group.layout().addLayout(progress_inner)

    progress_text = make_widget("Input", default_text='Awaiting Prompt', key='-PROGRESS_TEXT-', size=(20, 20))
    progress_text.setStyleSheet("background-color: light blue;")
    progress_bar = make_widget("ProgressBar", key='-PROGRESS-', size=(10, 20))
    query_count = make_widget("Input", default_text='0', key=text_to_key("query count"), size=(30, 20), disabled=True)

    progress_inner.addWidget(progress_text)
    progress_inner.addWidget(progress_bar)
    progress_inner.addWidget(query_count)

    h_layout.addWidget(progress_group)

    # query title frame
    query_title_group = make_widget("Frame", title='query title')
    query_title_inner = QHBoxLayout()
    query_title_group.layout().addLayout(query_title_inner)

    title_input = make_widget("Input", default_text="title of prompt", key=text_to_key('title input'), size=(30, 1))

    query_title_inner.addWidget(title_input)
    h_layout.addWidget(query_title_group)

    # response nav frame
    response_nav_group = make_widget("Frame", title="response nav")
    response_nav_inner = QHBoxLayout()
    response_nav_group.layout().addLayout(response_nav_inner)

    nav = get_left_right_nav(name='response text', section=False, push=False)
    response_nav_inner.addWidget(nav)

    h_layout.addWidget(response_nav_group)

    return progress_widget

def get_output_options():
    options_widget = QWidget()
    h_layout = QHBoxLayout()
    options_widget.setLayout(h_layout)

    submit_btn = make_widget("Button", button_text="SUBMIT QUERY", key="-SUBMIT_QUERY-")
    clear_requests = make_widget("Button", button_text="CLEAR REQUESTS", key='-CLEAR_REQUESTS-')
    clear_chunks = make_widget("Button", button_text="CLEAR CHUNKS", key='-CLEAR_CHUNKS-')
    gen_readme = make_widget("Button", button_text="GEN README", key='-GENERATE_README-')

    h_layout.addWidget(submit_btn)
    h_layout.addWidget(clear_requests)
    h_layout.addWidget(clear_chunks)
    h_layout.addWidget(gen_readme)

    return options_widget

# From prompt_tabs.py
prompt_tab_keys = ['request', 'prompt data', 'chunks', 'query', 'instructions']

def get_prompt_tabs(layout_specs={}, args={}):
    tab_group = make_widget("TabGroup", **args)

    for key in prompt_tab_keys:
        layout = layout_specs.get(key, [])
        tab = get_tab_layout(key, layout)
        tab_widget = QWidget()
        tab_layout = QVBoxLayout()
        tab_widget.setLayout(tab_layout)
        build_layout(tab['layout'], tab_layout)
        tab_group.addTab(tab_widget, tab['title'])

    return tab_group

def get_chunked_sections():
    chunk_widget = QWidget()
    v_layout = QVBoxLayout()
    chunk_widget.setLayout(v_layout)

    v_layout.addWidget(get_left_right_nav(name='chunk'))
    v_layout.addWidget(get_left_right_nav(name='chunk', section=False))

    frame = make_widget("Frame", title="chunk data")
    multi = get_right_click_multi(key=text_to_key('chunk sectioned data'), args={"enable_events":True, **expandable()})
    frame.layout().addWidget(multi)
    v_layout.addWidget(frame)

    return chunk_widget

def get_prompt_data_section():
    pd_widget = QWidget()
    v_layout = QVBoxLayout()
    pd_widget.setLayout(v_layout)

    v_layout.addWidget(get_left_right_nav(name='prompt_data'))

    frame = make_widget("Frame", title='prompt data')
    multi = get_right_click_multi(key=text_to_key('prompt_data data'), args={"enable_events":True, **expandable()})
    frame.layout().addWidget(multi)
    v_layout.addWidget(frame)

    return pd_widget

# Similarly for other sections...
# (To save space, implement similar for get_request_section, get_query_section, get_instructions)

# From gui_utilities.py (partial)
def utilities():
    tab_group = make_widget("TabGroup", **expandable(size=(int(0.4*window_width), window_height)))

    # Add tabs for SETTINGS, RESPONSES, Files, QUERY, urls, feedback
    # Implement each as QWidget with layout

    # Example for SETTINGS
    settings_tab = QWidget()
    settings_layout = QVBoxLayout()
    settings_tab.setLayout(settings_layout)
    # Add content from get_settings()
    tab_group.addTab(settings_tab, "SETTINGS")

    # Add other tabs similarly...

    return tab_group

# Main total layout from abstract_ai_gui_shared.py
def get_total_layout():
    main_widget = QWidget()
    main_layout = QVBoxLayout()
    main_widget.setLayout(main_layout)

    main_layout.addWidget(get_progress_frame())
    main_layout.addWidget(get_output_options())

    h_layout = QHBoxLayout()
    main_layout.addLayout(h_layout)

    # Prompt tabs with specs
    layouts = {"query": get_query_section(), "request": get_request_section(), "prompt data": get_prompt_data_section(),
               "instructions": get_instructions(), "chunks": get_chunked_sections()}
    prompt = get_prompt_tabs(layouts, args=expandable(size=(int(0.2*window_width), window_height)))
    h_layout.addWidget(prompt)

    util_col = get_column([utilities()])
    h_layout.addWidget(util_col)

    return main_widget

# Other functions like env_section, swap_commands, currentTokenInfo, etc., can be converted similarly.

# Main app
class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Abstract AI GUI - PyQt Version")
        self.setGeometry(100, 100, window_width, window_height)

        central_widget = get_total_layout()
        self.setCentralWidget(central_widget)

        # Connect signals as needed (e.g., buttons to functions)
        # Example: find button by objectName and connect

if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec())



#from ..dependencies import *
# updates instructions manager with environment values from GUI      
def update_instruction_mgr(self)->None:
    """
    updates the Instruction Manager with the environment values from the GUI. If the '-instructions-'
    key in the UI is set, it also updates the Instruction Manager's attributes for each instruction key.
    """
    spec_num = self.get_spec_section_number_display('request')
    self.instruction_data_list[spec_num]['text_values']={"api_response":"place response to prompt here"}
    for key in self.instruction_pre_keys:
        bool_key = text_to_key(text=key,section="BOOL")
        text_key = text_to_key(text=key,section="TEXT")
        self.instruction_data_list[spec_num]['bool_values'][key]=self.window_mgr.get_from_value(bool_key)
        if self.window_mgr.get_from_value(bool_key):
            self.instruction_data_list[spec_num]['text_values'][key]=self.window_mgr.get_from_value(text_key)
    instructions_display_text_value=""
    if self.instruction_data_list[spec_num]['bool_values']["instructions"]:
        instructions_display_text_value = self.instruction_mgr.get_instructions_text(self.instruction_data_list[spec_num]['text_values'])
    self.instruction_data_list[spec_num]['text']=instructions_display_text_value
    self.window_mgr.update_value(key="-INSTRUCTIONS_TEXT-",value=self.instruction_data_list[spec_num]['text'])
    self.prompt_mgr.update_request_and_prompt_data(instruction_data = self.instruction_data_list)
    
def update_bool_instructions(self)->None:
    spec_num = self.get_spec_section_number_display('request')
    self.instruction_data_list[spec_num]['bool_values']['api_response']=True
    for key in self.instruction_pre_keys:
        bool_key = text_to_key(text=key,section="BOOL")
        value=False
        if key in self.instruction_data_list[spec_num]['bool_values']:
            value = self.instruction_data_list[spec_num]['bool_values'][key]
        self.window_mgr.update_value(key=bool_key,value=value)
    self.window_mgr.update_value(key="-INSTRUCTIONS_TEXT-",value=self.instruction_data_list[spec_num]['text'])
    
def restore_instruction_defaults(self)->None:
    defaults = self.instruction_mgr.default_instructions
    for key in self.instruction_pre_keys:
        text_key = text_to_key(text=key,section="TEXT")
        if key in defaults:
            self.window_mgr.update_value(key=text_key,value=defaults[key]['instruction'])

def delegate_instruction_text(self)->None:
    """
    updates the instructions section in the GUI with the values in the latest response. If the instruction
    key isn't in the GUI's value list, it appends the instruction to the '-other-' section of feedback in the GUI.
    """
    for key in self.instruction_pre_keys:
        value = get_any_value(safe_json_loads(self.last_response_content),key)
        setattr(self,'last_'+key,value)
        self.unlist_self(['last_'+key])
        instruction_key = text_to_key(key,section='feedback')
        if instruction_key not in self.window_mgr.get_values():
            if value:
                self.append_output(text_to_key(text='other',section='feedback'),f"{key}: {value}"+'\n')
        else:
            self.window_mgr.update_value(key=instruction_key,value=value)

def instruction_event_check(self)->bool:
    """
    checks if there was an instruction-related event and updates the Instruction Manager if there was one.
    It then updates the Prompt Manager with the latest instructions.
    """
    if self.event in self.instruction_bool_keys:
        self.update_instruction_mgr()

    else:
        return False
    self.update_query_display()
    return True



# controller.py
import asyncio
from PyQt5 import QtCore, QtWidgets
import sys


from ..gpt_classes import update_instruction_mgr,ApiManager,ModelManager,PromptManager,InstructionManager,ResponseManager,update_instruction_mgr
from ..gpt_classes.instruction_section import update_instruction_mgr
import json
import openai

class Controller(QtWidgets.QMainWindow):
    def __init__(self):
        super().__init__()

        # 1) Build and show the GUI
        self.gui = MainWindow()
        self.gui.show()

        # 2) Instantiate managers
        self.instruction_mgr = InstructionManager()
        self.model_mgr = ModelManager(default_selection=True)
        self.prompt_mgr = PromptManager(
            instruction_mgr=self.instruction_mgr,
            model_mgr=self.model_mgr
        )
        self.api_mgr = ApiManager()
        self.response_mgr = ResponseManager(
            prompt_mgr=self.prompt_mgr,
            api_mgr=self.api_mgr
        )
        self.window_mgr = WindowManager(self.gui)
        self.navigation_mgr = NavigationManager()
        self.history_mgr = HistoryManager()
        self.update_instruction_mgr = update_instruction_mgr
        # 3) Initialize data structures
        self.instruction_data_list = self.instruction_mgr.create_empty_instruction_list()
        self.instruction_pre_keys = list(self.instruction_mgr.default_instructions.keys())
        self.prompt_data_list = [""] * 10
        self.request_data_list = [""] * 10
        self.script_event_js = {"found": None, "section": None}
        self.display_number_tracker = {
            "request": 0,
            "prompt_data": 0,
            "chunk_number": 0,
            "query": 0,
            "chunk_section_number": 0,
        }

        # Chunk-related defaults
        self.chunk_display_keys = ["-CHUNK_TEXT-", "-CHUNK_TOKEN_COUNT-", "-CHUNK_TOKEN_PERCENTAGE-"]
        self.sectioned_chunk_data_key = "-CHUNK_TEXT-"
        self.chunk_history_name = "PROMPT_DATA_HISTORY"
        self.request_history_name = "REQUEST_DATA_HISTORY"
        self.chunk_type = None
        self.url_chunk_type = "URL"
        self.token_percentage_dropdowns = ["-PROMPT_PERCENTAGE-", "-COMPLETION_PERCENTAGE-"]
        self.instruction_bool_keys = [text_to_key(k, section="BOOL") for k in self.instruction_pre_keys]

        # 4) Set up asyncio event loop for PyQt5
        self.loop = asyncio.get_event_loop()
        if not self.loop.is_running():
            self.loop.run_until_complete(asyncio.sleep(0))

        # 5) Wire up signals
        self._wire_instruction_signals()
        self._wire_prompt_signals()
        self._wire_output_signals()
        self._wire_settings_signals()

        # 6) Initialize model selection in GUI
        self._update_model_combo()

    def _update_model_combo(self):
        """Populate model combo box with available models."""
        model_combo = self.gui.utilities_tabs.findChild(QtWidgets.QComboBox, None, options=QtCore.Qt.FindChildrenRecursively)
        if model_combo and hasattr(self.model_mgr, 'all_model_names'):
            model_combo.clear()
            model_combo.addItems(self.model_mgr.all_model_names)
            current_model = self.model_mgr.selected_model_name
            model_combo.setCurrentText(current_model)

    def _wire_instruction_signals(self):
        """Wire signals for instruction checkboxes and text fields."""
        for key in self.instruction_pre_keys:
            bool_key = text_to_key(text=key, section="BOOL")
            text_key = text_to_key(text=key, section="TEXT")

            checkbox = self.window_mgr.get_widget(bool_key, widget_type=QtWidgets.QCheckBox)
            if checkbox:
                checkbox.toggled.connect(lambda checked, k=key: self._on_instruction_bool_changed(k))

            if key in self.instruction_mgr.default_instructions:
                text_edit = self.window_mgr.get_widget(text_key, widget_type=QtWidgets.QTextEdit)
                if text_edit:
                    text_edit.textChanged.connect(lambda k=key: self._on_instruction_text_changed(k))

        main_instr_cb = self.window_mgr.get_widget(text_to_key("instructions", section="BOOL"), QtWidgets.QCheckBox)
        if main_instr_cb:
            main_instr_cb.toggled.connect(self.update_instruction_mgr)

    def _wire_prompt_signals(self):
        """Wire signals for prompt and request text edits."""
        request_edit = self.gui.prompt_tabs.findChild(QtWidgets.QTextEdit, "request_data")
        if request_edit:
            request_edit.textChanged.connect(self._on_request_text_changed)

        prompt_data_edit = self.gui.prompt_tabs.findChild(QtWidgets.QTextEdit, "prompt_data_data")
        if prompt_data_edit:
            prompt_data_edit.textChanged.connect(self._on_prompt_data_text_changed)

        add_query_btn = self.window_mgr.get_widget("-ADD_QUERY-", QtWidgets.QPushButton)
        if add_query_btn:
            add_query_btn.clicked.connect(self._on_add_query)

    def _wire_output_signals(self):
        """Wire signals for output option buttons."""
        for btn in self.gui.output_options.findChildren(QtWidgets.QPushButton):
            text = btn.text().upper()
            if text == "SUBMIT QUERY":
                btn.clicked.connect(self.on_submit_query)
            elif text == "CLEAR CHUNKS":
                btn.clicked.connect(self.on_clear_chunks)
            elif text == "CLEAR REQUESTS":
                btn.clicked.connect(self.on_clear_requests)
            elif text == "GEN README":
                btn.clicked.connect(self.on_gen_readme)

    def _wire_settings_signals(self):
        """Wire signals for settings tab, e.g., model selection and token percentage."""
        settings_tab = self.gui.utilities_tabs.findChild(QtWidgets.QScrollArea)
        if settings_tab:
            model_combo = settings_tab.findChild(QtWidgets.QComboBox, None, options=QtCore.Qt.FindChildrenRecursively)
            if model_combo:
                model_combo.currentTextChanged.connect(self._on_model_changed)

            token_combos = settings_tab.findChildren(QtWidgets.QComboBox)
            for combo in token_combos:
                if combo.objectName() in self.token_percentage_dropdowns:
                    combo.currentTextChanged.connect(self._on_token_percentage_changed)

    def _on_instruction_bool_changed(self, key: str):
        self.update_instruction_mgr()
        self._update_prompt_manager()

    def _on_instruction_text_changed(self, key: str):
        self.update_instruction_mgr()
        self._update_prompt_manager()

    def _on_request_text_changed(self):
        self.update_request_data_list()
        self.update_query_display()
        self._update_prompt_manager()
        self.fill_lists()

    def _on_prompt_data_text_changed(self):
        self.update_prompt_data_list()
        self.update_query_display()
        self._update_prompt_manager()
        self.fill_lists()

    def _on_add_query(self):
        self.event = "-ADD_QUERY-"
        self.prompt_request_event_check()
        self.fill_lists()

    def _on_model_changed(self, model_name: str):
        self.model_mgr.selected_model_name = model_name
        self.model_mgr.selected_endpoint = self.model_mgr._get_endpoint_by_model(model_name)
        self.model_mgr.selected_max_tokens = self.model_mgr._get_max_tokens_by_model(model_name)
        self._update_prompt_manager()
        self.statusBar().showMessage(f"Model changed to {model_name}", 3000)

    def _on_token_percentage_changed(self):
        completion_combo = self.window_mgr.get_widget("-COMPLETION_PERCENTAGE-", QtWidgets.QComboBox)
        if completion_combo:
            completion_percentage = int(completion_combo.currentText())
            self.prompt_mgr.update_request_and_prompt_data(
                completion_percentage=completion_percentage
            )
            self.statusBar().showMessage(f"Completion percentage set to {completion_percentage}%", 2000)

    def _update_prompt_manager(self):
        """Update PromptManager with current GUI data."""
        request_edit = self.gui.prompt_tabs.findChild(QtWidgets.QTextEdit, "request_data")
        prompt_data_edit = self.gui.prompt_tabs.findChild(QtWidgets.QTextEdit, "prompt_data_data")
        request_data = request_edit.toPlainText() if request_edit else ""
        prompt_data = prompt_data_edit.toPlainText() if prompt_data_edit else ""
        self.prompt_mgr.update_request_and_prompt_data(
            model_mgr=self.model_mgr,
            instruction_mgr=self.instruction_mgr,
            request_data=[request_data],
            prompt_data=[prompt_data],
            instruction_data=[self.instruction_mgr.get_instructions()],
        )
        self.response_mgr.prompt_mgr = self.prompt_mgr

    async def _async_submit_query(self):
        """Asynchronously submit query using ResponseManager."""
        try:
            outputs = await self.response_mgr.initial_query()
            for output in outputs:
                response_content = output.get("response", {}).get("choices", [{}])[0].get("message", {}).get("content", "")
                if isinstance(response_content, str):
                    try:
                        response_content = json.loads(response_content)
                    except json.JSONDecodeError:
                        pass

                response_edit = self.gui.utilities_tabs.findChild(QtWidgets.QTextEdit, "response_edit")
                if response_edit:
                    response_edit.clear()
                    response_edit.setPlainText(json.dumps(response_content, indent=2) if isinstance(response_content, dict) else str(response_content))

                self.last_response_content = output
                self.delegate_instruction_text()

            self.statusBar().showMessage("Query submitted successfully.", 3000)
        except Exception as e:
            self.statusBar().showMessage(f"Query Error: {str(e)}", 5000)

    def on_submit_query(self):
        """Handle Submit Query button click."""
        final_prompt = self.window_mgr.get_from_value("-QUERY-")
        if not final_prompt:
            self.statusBar().showMessage("No prompt to submit.", 3000)
            return

        # Run async query in the event loop
        asyncio.ensure_future(self._async_submit_query())

    def on_clear_chunks(self):
        self.event = "-CLEAR_CHUNKS-"
        self.chunk_event_check()
        self.statusBar().showMessage("Chunks cleared.", 2000)

    def on_clear_requests(self):
        self.request_data_list = [""] * 10
        self.update_request_data_display()
        self._update_prompt_manager()
        self.statusBar().showMessage("Requests cleared.", 2000)

    def on_gen_readme(self):
        # Placeholder for GEN README functionality
        self.statusBar().showMessage("Generating README (not implemented).", 2000)

    

    def fill_lists(self):
        pass

def main():
    app = QtWidgets.QApplication(sys.argv)
    controller = Controller()
    sys.exit(app.exec_())
main()



    IMAGE_INSTRUCTION_KEYS = {
        "alt_text": {
            "instruction": (
                "Provide a concise description of what the image visibly contains. "
                "Avoid phrases like 'image of'. Keep under 160 characters."
            ),
            "example": "Abstract Endeavors logo representing innovative wealth management solutions",
            "default": True
        },
        "title_tag": {
            "instruction": (
                "Generate a clean and descriptive title for the image, suitable for HTML title tags "
                "and search engine previews."
            ),
            "example": "Abstract Endeavors - The Future of Wealth Management",
            "default": True
        },
        "longdesc": {
            "instruction": (
                "Write a detailed, multi-sentence description summarizing the visual appearance, "
                "purpose, and context of the image."
            ),
            "example": (
                "Explore the Abstract Endeavors logo, a symbol of innovative approaches in wealth "
                "management, reflecting a new perspective on financial growth and strategy."
            ),
            "default": True
        },
        "caption": {
            "instruction": (
                "Provide a short, natural-language caption that could appear under the image in an article."
            ),
            "example": "Abstract Endeavors - The Future of Wealth Management",
            "default": True
        },
        "keywords": {
            "instruction": (
                "Provide a comma-separated list of SEO keywords relevant to the image. "
                "Avoid special characters and include 7-15 items."
            ),
            "example": "wealth management, innovative logo, finance strategy, branding, modern design",
            "default": True
        },
    }



# cli.py
import argparse,sys,os,subprocess
from abstract_utilities.robust_reader import read_file_as_text

def _run_gui():
    from gui_frontend import main as gui_main
    gui_main()

def _run_flask():
    # Launch Flask in‐process (blocking)
    from flask import create_app
    app = create_app()
    app.run(host='127.0.0.1', port=7820)

def _copy_file(path):
    try:
        text = read_file_as_text(path)
        # Use xclip if PyQt is not available
        import subprocess
        p = subprocess.Popen(['xclip','-selection','clipboard'], stdin=subprocess.PIPE)
        p.communicate(input=text.encode('utf-8'))
        print(f"Copied {path!r} to clipboard.")
    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)

def client_main():
    parser = argparse.ArgumentParser(prog='clipit')
    parser.add_argument('--mode', choices=['gui','flask','copy'], default='copy',
                        help="Choose 'gui' (PyQt), 'flask' (HTTP), or 'copy' (CLI)")
    parser.add_argument('--file', type=str,
                        help="When mode=copy, path to file to copy to clipboard")
    args = parser.parse_args()

    if args.mode == 'gui':
        _run_gui()
    elif args.mode == 'flask':
        _run_flask()
    else:  # copy
        if not args.file:
            print("Error: --file is required when mode=copy", file=sys.stderr)
            sys.exit(1)
        _copy_file(args.file)

